/* IMPORT */
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const globby = require("globby");
const string_matches_1 = require("string-matches");
const config_1 = require("../config");
const consts_1 = require("../consts");
const file_1 = require("./file");
/* EMBEDDED */
const Embedded = {
    get(rootPaths) {
        return __awaiter(this, void 0, void 0, function* () {
            const filePaths = yield Embedded.getFilePaths(rootPaths);
            if (!filePaths.length)
                return;
            const todos = yield Embedded.getFilesTodos(filePaths, consts_1.default.regexes.todoEmbedded), content = yield Embedded.renderTodos(todos);
            return content;
        });
    },
    getFilePaths(rootPaths) {
        return __awaiter(this, void 0, void 0, function* () {
            rootPaths = _.castArray(rootPaths);
            const config = config_1.default.get(), { include, exclude } = config.embedded;
            let filePaths = [];
            for (let rootPath of rootPaths) {
                const rootFilePaths = yield globby(include, { cwd: rootPath, ignore: exclude, absolute: true });
                filePaths = filePaths.concat(rootFilePaths);
            }
            return filePaths;
        });
    },
    getFilesTodos(filePaths, regex) {
        return __awaiter(this, void 0, void 0, function* () {
            const todos = {}; // { [TYPE] => { [FILE] => [{ LINE, NR }] } }
            yield Promise.all(filePaths.map((filePath) => __awaiter(this, void 0, void 0, function* () {
                const content = yield file_1.default.read(filePath);
                if (!content)
                    return;
                const lines = content.split('\n');
                lines.forEach((line, lineNr) => {
                    const matches = string_matches_1.default(line, regex);
                    matches.forEach(match => {
                        const type = match[1];
                        if (!todos[type])
                            todos[type] = {};
                        if (!todos[type][filePath])
                            todos[type][filePath] = [];
                        todos[type][filePath].push({ line, lineNr });
                    });
                });
            })));
            return todos;
        });
    },
    renderTodos(todos) {
        const config = config_1.default.get(), { indentation, embedded: { groupByFile }, symbols: { box } } = config, lines = [];
        /* LINES */
        const types = Object.keys(todos).sort();
        types.forEach(type => {
            const files = todos[type], filePaths = Object.keys(files).sort(), typeLines = [];
            filePaths.forEach(filePath => {
                const todos = files[filePath], normalizedFilePath = `/${_.trimStart(filePath, '/')}`;
                if (groupByFile) {
                    typeLines.push(`${indentation}@file://${normalizedFilePath}`);
                }
                todos.forEach(({ line, lineNr }) => {
                    typeLines.push(`${indentation}${groupByFile ? indentation : ''}${box} ${_.trimStart(line)} @file://${normalizedFilePath}#${lineNr + 1}`);
                });
            });
            lines.push(`${type}:`);
            lines.push(...typeLines);
        });
        return lines.length ? `${lines.join('\n')}\n` : '';
    }
};
/* EXPORT */
exports.default = Embedded;
//# sourceMappingURL=embedded.js.map